@@ -262,6 +262,33 @@ get_monitor_transform (GfMonitorManager *monitor_manager,
   return gf_monitor_transform_from_orientation (orientation);
 }
 
+static void
+scale_logical_monitor_width (GfLogicalMonitorLayoutMode  layout_mode,
+                             float                       scale,
+                             int                         mode_width,
+                             int                         mode_height,
+                             int                        *width,
+                             int                        *height)
+{
+  switch (layout_mode)
+    {
+      case GF_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
+        *width = (int) roundf (mode_width / scale);
+        *height = (int) roundf (mode_height / scale);
+        return;
+
+      case GF_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
+        *width = mode_width;
+        *height = mode_height;
+        return;
+
+      default:
+        break;
+    }
+
+  g_assert_not_reached ();
+}
+
 static GfLogicalMonitorConfig *
 create_preferred_logical_monitor_config (GfMonitorManager           *monitor_manager,
                                          GfMonitor                  *monitor,
@@ -279,17 +306,12 @@ create_preferred_logical_monitor_config (GfMonitorManager           *monitor_man
   mode = gf_monitor_get_preferred_mode (monitor);
   gf_monitor_mode_get_resolution (mode, &width, &height);
 
-  switch (layout_mode)
-    {
-      case GF_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
-        width = (int) roundf (width / scale);
-        height = (int) roundf (height / scale);
-        break;
-
-      case GF_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
-      default:
-        break;
-    }
+  scale_logical_monitor_width (layout_mode,
+                               scale,
+                               width,
+                               height,
+                               &width,
+                               &height);
 
   monitor_config = gf_monitor_config_new (monitor, mode);
   transform = get_monitor_transform (monitor_manager, monitor);
@@ -318,10 +340,125 @@ create_preferred_logical_monitor_config (GfMonitorManager           *monitor_man
   return logical_monitor_config;
 }
 
+static GfLogicalMonitorConfig *
+find_monitor_config (GfMonitorsConfig *config,
+                     GfMonitor        *monitor,
+                     GfMonitorMode    *monitor_mode)
+{
+  int mode_width;
+  int mode_height;
+  GList *l;
+
+  gf_monitor_mode_get_resolution (monitor_mode, &mode_width, &mode_height);
+
+  for (l = config->logical_monitor_configs; l; l = l->next)
+    {
+      GfLogicalMonitorConfig *logical_monitor_config;
+      GList *l_monitor;
+
+      logical_monitor_config = l->data;
+
+      for (l_monitor = logical_monitor_config->monitor_configs; l_monitor; l_monitor = l_monitor->next)
+        {
+          GfMonitorConfig *monitor_config;
+          GfMonitorModeSpec *mode_spec;
+
+          monitor_config = l_monitor->data;
+          mode_spec = gf_monitor_mode_get_spec (monitor_mode);
+
+          if (gf_monitor_spec_equals (gf_monitor_get_spec (monitor), monitor_config->monitor_spec) &&
+              gf_monitor_mode_spec_has_similar_size (mode_spec, monitor_config->mode_spec))
+            return logical_monitor_config;
+        }
+    }
+
+  return NULL;
+}
+
+static gboolean
+get_last_scale_for_monitor (GfMonitorConfigManager *config_manager,
+                            GfMonitor              *monitor,
+                            GfMonitorMode          *monitor_mode,
+                            float                  *out_scale)
+{
+  GList *configs;
+  GList *l;
+
+  configs = NULL;
+
+  if (config_manager->current_config != NULL)
+    configs = g_list_append (configs, config_manager->current_config);
+
+  configs = g_list_concat (configs, g_list_copy (config_manager->config_history.head));
+
+  for (l = configs; l; l = l->next)
+    {
+      GfMonitorsConfig *config;
+      GfLogicalMonitorConfig *logical_monitor_config;
+
+      config = l->data;
+      logical_monitor_config = find_monitor_config (config,
+                                                    monitor,
+                                                    monitor_mode);
+
+      if (logical_monitor_config != NULL)
+        {
+          *out_scale = logical_monitor_config->scale;
+          g_list_free (configs);
+
+          return TRUE;
+        }
+    }
+
+  g_list_free (configs);
+
+  return FALSE;
+}
+
+
+static float
+compute_scale_for_monitor (GfMonitorConfigManager *config_manager,
+                           GfMonitor              *monitor,
+                           GfMonitor              *primary_monitor)
+
+{
+  GfMonitorManager *monitor_manager;
+  GfMonitor *target_monitor;
+  GfMonitorManagerCapability capabilities;
+  GfLogicalMonitorLayoutMode layout_mode;
+  GfMonitorMode *monitor_mode;
+  float scale;
+
+  monitor_manager = config_manager->monitor_manager;
+  target_monitor = monitor;
+  capabilities = gf_monitor_manager_get_capabilities (monitor_manager);
+
+  if ((capabilities & GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED) &&
+      primary_monitor != NULL)
+    {
+      target_monitor = primary_monitor;
+    }
+
+  layout_mode = gf_monitor_manager_get_default_layout_mode (monitor_manager);
+  monitor_mode = gf_monitor_get_preferred_mode (target_monitor);
+
+  if (get_last_scale_for_monitor (config_manager,
+                                  target_monitor,
+                                  monitor_mode,
+                                  &scale))
+    return scale;
+
+  return gf_monitor_manager_calculate_monitor_mode_scale (monitor_manager,
+                                                          layout_mode,
+                                                          target_monitor,
+                                                          monitor_mode);
+}
+
 static GfMonitorsConfig *
 create_for_switch_config_all_mirror (GfMonitorConfigManager *config_manager)
 {
   GfMonitorManager *monitor_manager = config_manager->monitor_manager;
+  GfMonitor *primary_monitor;
   GfLogicalMonitorLayoutMode layout_mode;
   GfLogicalMonitorConfig *logical_monitor_config = NULL;
   GList *logical_monitor_configs;
@@ -333,6 +470,14 @@ create_for_switch_config_all_mirror (GfMonitorConfigManager *config_manager)
   GList *monitors;
   GList *l;
   GfMonitorsConfig *monitors_config;
+  int width;
+  int height;
+
+  primary_monitor = find_primary_monitor (monitor_manager,
+                                          MONITOR_MATCH_ALLOW_FALLBACK);
+
+  if (primary_monitor == NULL)
+    return NULL;
 
   layout_mode = gf_monitor_manager_get_default_layout_mode (monitor_manager);
   monitors = gf_monitor_manager_get_monitors (monitor_manager);
@@ -406,25 +551,32 @@ create_for_switch_config_all_mirror (GfMonitorConfigManager *config_manager)
       if (!mode)
         continue;
 
-      scale = gf_monitor_manager_calculate_monitor_mode_scale (monitor_manager,
-                                                               layout_mode,
-                                                               l_monitor,
-                                                               mode);
+      scale = compute_scale_for_monitor (config_manager,
+                                         l_monitor,
+                                         primary_monitor);
 
       best_scale = MAX (best_scale, scale);
       monitor_configs = g_list_prepend (monitor_configs, gf_monitor_config_new (l_monitor, mode));
     }
 
+  scale_logical_monitor_width (layout_mode,
+                               best_scale,
+                               common_mode_w,
+                               common_mode_h,
+                               &width,
+                               &height);
+
   logical_monitor_config = g_new0 (GfLogicalMonitorConfig, 1);
   *logical_monitor_config = (GfLogicalMonitorConfig) {
     .layout = (GfRectangle) {
       .x = 0,
       .y = 0,
-      .width = common_mode_w,
-      .height = common_mode_h
+      .width = width,
+      .height = height
     },
     .scale = best_scale,
-    .monitor_configs = monitor_configs
+    .monitor_configs = monitor_configs,
+    .is_primary = TRUE
   };
 
   logical_monitor_configs = g_list_append (NULL, logical_monitor_config);
@@ -487,35 +639,6 @@ verify_suggested_monitors_config (GList *logical_monitor_configs)
   return TRUE;
 }
 
-static float
-compute_scale_for_monitor (GfMonitorManager *monitor_manager,
-                           GfMonitor        *monitor,
-                           GfMonitor        *primary_monitor)
-
-{
-  GfMonitor *target_monitor;
-  GfMonitorManagerCapability capabilities;
-  GfLogicalMonitorLayoutMode layout_mode;
-  GfMonitorMode *monitor_mode;
-
-  target_monitor = monitor;
-  capabilities = gf_monitor_manager_get_capabilities (monitor_manager);
-
-  if ((capabilities & GF_MONITOR_MANAGER_CAPABILITY_GLOBAL_SCALE_REQUIRED) &&
-      primary_monitor != NULL)
-    {
-      target_monitor = primary_monitor;
-    }
-
-  layout_mode = gf_monitor_manager_get_default_layout_mode (monitor_manager);
-  monitor_mode = gf_monitor_get_preferred_mode (target_monitor);
-
-  return gf_monitor_manager_calculate_monitor_mode_scale (monitor_manager,
-                                                          layout_mode,
-                                                          target_monitor,
-                                                          monitor_mode);
-}
-
 static GfMonitorsConfig *
 create_monitors_config (GfMonitorConfigManager *config_manager,
                         MonitorMatchRule        match_rule,
@@ -571,7 +694,7 @@ create_monitors_config (GfMonitorConfigManager *config_manager,
             break;
         }
 
-      scale = compute_scale_for_monitor (monitor_manager,
+      scale = compute_scale_for_monitor (config_manager,
                                          monitor,
                                          primary_monitor);
 
@@ -668,7 +791,9 @@ clone_monitor_config_list (GList *configs_in)
       *config_out = (GfMonitorConfig) {
         .monitor_spec = gf_monitor_spec_clone (config_in->monitor_spec),
         .mode_spec = g_memdup2 (config_in->mode_spec, sizeof (GfMonitorModeSpec)),
-        .enable_underscanning = config_in->enable_underscanning
+        .enable_underscanning = config_in->enable_underscanning,
+        .has_max_bpc = config_in->has_max_bpc,
+        .max_bpc = config_in->max_bpc
       };
 
       configs_out = g_list_append (configs_out, config_out);
@@ -1022,7 +1147,9 @@ assign_monitor_crtc (GfMonitor          *monitor,
     .output = output,
     .is_primary = assign_output_as_primary,
     .is_presentation = assign_output_as_presentation,
-    .is_underscanning = data->monitor_config->enable_underscanning
+    .is_underscanning = data->monitor_config->enable_underscanning,
+    .has_max_bpc = data->monitor_config->has_max_bpc,
+    .max_bpc = data->monitor_config->max_bpc
   };
 
   g_ptr_array_add (data->crtc_assignments, crtc_assignment);
@@ -1061,7 +1188,7 @@ assign_monitor_crtcs (GfMonitorManager        *manager,
   if (!monitor_mode)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Invalid mode %dx%d (%f) for monitor '%s %s'",
+                   "Invalid mode %dx%d (%.3f) for monitor '%s %s'",
                    monitor_mode_spec->width, monitor_mode_spec->height,
                    (gdouble) monitor_mode_spec->refresh_rate,
                    monitor_spec->vendor, monitor_spec->product);
