@@ -49,6 +49,9 @@ typedef struct
 
   gboolean      is_underscanning;
 
+  gboolean      has_max_bpc;
+  unsigned int  max_bpc;
+
   int           backlight;
 } GfOutputPrivate;
 
@@ -311,6 +314,11 @@ gf_output_assign_crtc (GfOutput                 *self,
   priv->is_primary = output_assignment->is_primary;
   priv->is_presentation = output_assignment->is_presentation;
   priv->is_underscanning = output_assignment->is_underscanning;
+
+  priv->has_max_bpc = output_assignment->has_max_bpc;
+
+  if (priv->has_max_bpc)
+    priv->max_bpc = output_assignment->max_bpc;
 }
 
 void
@@ -368,6 +376,10 @@ gf_output_is_laptop (GfOutput *output)
       case GF_CONNECTOR_TYPE_HDMIB:
       case GF_CONNECTOR_TYPE_TV:
       case GF_CONNECTOR_TYPE_VIRTUAL:
+      case GF_CONNECTOR_TYPE_DPI:
+      case GF_CONNECTOR_TYPE_WRITEBACK:
+      case GF_CONNECTOR_TYPE_SPI:
+      case GF_CONNECTOR_TYPE_USB:
       default:
         break;
     }
@@ -435,6 +447,20 @@ gf_output_is_underscanning (GfOutput *self)
   return priv->is_underscanning;
 }
 
+gboolean
+gf_output_get_max_bpc (GfOutput     *self,
+                       unsigned int *max_bpc)
+{
+  GfOutputPrivate *priv;
+
+  priv = gf_output_get_instance_private (self);
+
+  if (priv->has_max_bpc && max_bpc != NULL)
+    *max_bpc = priv->max_bpc;
+
+  return priv->has_max_bpc;
+}
+
 void
 gf_output_set_backlight (GfOutput *self,
                          int       backlight)
