@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Alberts Muktupāvels
+ * Copyright (C) 2015-2020 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,15 +18,17 @@
 #ifndef GF_INPUT_SOURCES_H
 #define GF_INPUT_SOURCES_H
 
-#include <glib-object.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-#define GF_TYPE_INPUT_SOURCES gf_input_sources_get_type ()
+#define GF_TYPE_INPUT_SOURCES (gf_input_sources_get_type ())
 G_DECLARE_FINAL_TYPE (GfInputSources, gf_input_sources,
                       GF, INPUT_SOURCES, GObject)
 
-GfInputSources *gf_input_sources_new (void);
+GfInputSources *gf_input_sources_new           (void);
+
+GtkWidget      *gf_input_sources_create_button (GfInputSources *self);
 
 G_END_DECLS
 
